Package: netrankr
Type: Package
Title: Analyzing Partial Rankings in Networks
Version: 1.2.4
Authors@R: c(
    person("David", "Schoch", email = "david@schochastics.net", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-2952-4812")),
    person("Julian","Müller", email = "julian.mueller@gess.ethz.ch", role = "ctb"))
Description: Implements methods for centrality related analyses of networks. 
    While the package includes the possibility to build more than 20 indices, 
    its main focus lies on index-free assessment of centrality via partial 
    rankings obtained by neighborhood-inclusion or positional dominance. These 
    partial rankings can be analyzed with different methods, including 
    probabilistic methods like computing expected node ranks and relative 
    rank probabilities (how likely is it that a node is more central than another?).
    The methodology is described in depth in the vignettes and in
    Schoch (2018) <doi:10.1016/j.socnet.2017.12.003>.
URL: https://github.com/schochastics/netrankr/,
        https://schochastics.github.io/netrankr/
BugReports: https://github.com/schochastics/netrankr/issues
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.0.1)
Imports: igraph (>= 1.0.1), Rcpp (>= 0.12.8), Matrix
LinkingTo: Rcpp,RcppArmadillo
RoxygenNote: 7.3.2
Suggests: knitr, rmarkdown, magrittr, testthat, shiny (>= 0.13), miniUI
        (>= 0.1.1), rstudioapi (>= 0.5), covr
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2025-02-05 09:14:09 UTC; david
Author: David Schoch [aut, cre] (<https://orcid.org/0000-0003-2952-4812>),
  Julian Müller [ctb]
Maintainer: David Schoch <david@schochastics.net>
Repository: CRAN
Date/Publication: 2025-02-05 11:10:05 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 04:07:54 UTC; unix
Archs: netrankr.so.dSYM
